#include <Wire.h>
#include "bitmapdata.h"
static uint8_t oledAddr = 0x3C;     // SSD1306のI2Cアドレス
static uint8_t oldCommand = 0x80;   // SSD1306の1Byteコマンド
static uint8_t oldCommands = 0x00;  // SSD1306の複数Byteコマンド
static uint8_t oldData = 0x40;      // SSD1306のデータバイト
static uint8_t oledInitCommands[] = { // SSD1306の初期化コマンド群
  0xC8,         // 上下の描画方向
  0xA1,         // 左右の描画方向
  0xA8,0x3F,    // 画面の解像度
  0xD3,0x00,    // 縦方向のオフセット
  0x40,         // 縦方向の描画開始位置
  0xA6,         // 表示方法 - A7にすると画面反転
  0x8D, 0x14    // OLEDパネルの供給電圧と思われる 14hに設定しないとダメらしい
};

uint8_t oledWidth = 128;  // OLEDの幅(ピクセル数)
uint8_t oledHeight = 64;  // OLEDの高さ(ピクセル数)
uint8_t oledPages = 8;    // OLEDのページ数(ピクセル数÷８)

// OLEDに1バイトコマンドを送信
void oledSendCommand(uint8_t command) {
  Wire.beginTransmission(oledAddr);
  Wire.write(oldCommand);
  Wire.write(command);
  Wire.endTransmission();
}

// OLEDに2バイトコマンドを送信
void oledSendCommand2(uint8_t command1,uint8_t command2) {
  Wire.beginTransmission(oledAddr);
  Wire.write(oldCommands);
  Wire.write(command1);
  Wire.write(command2);
  Wire.endTransmission();
}

// OLEDに3バイトコマンドを送信
void oledSendCommand3(uint8_t command1,uint8_t command2,uint8_t command3) {
  Wire.beginTransmission(oledAddr);
  Wire.write(oldCommands);
  Wire.write(command1);
  Wire.write(command2);
  Wire.write(command3);
  Wire.endTransmission();
}

// OLEDに複数のコマンドを送信::Wireクラスの制限により最大32Byte
// 長さのチェックはしてないのでプログラマ責任で上手い事やってちょ
void oledSendCommands(int commandLen,uint8_t commands[]) {
  Wire.beginTransmission(oledAddr);
  Wire.write(oldCommands);
  for (int i = 0; i < commandLen; i++) {
    Wire.write(commands[i]);
  }
   Wire.endTransmission();
}

// 画面を特定のビットバターンで埋める::画面の初期化用
void oledFillData(uint8_t fillData) {
  for (uint8_t m = 0; m < oledPages; m++) {
    oledSendCommand3(0xB0 | (m & 0x07) , 0x10 , 0x00) ;

    Wire.beginTransmission(oledAddr);
    Wire.write(oldData);
    for (uint8_t x = 0; x < oledWidth; x++) {
      if (Wire.write(fillData) == 0) {
        Wire.endTransmission();
        Wire.beginTransmission(oledAddr);
        Wire.write(oldData);
        Wire.write(fillData);
      }
    }
    Wire.endTransmission();
  }
  oledSendCommand3(0xB0 , 0x10 , 0x00) ;
}

// BITマップを表示する::oledFillDataの可変データ型なので基本oledFillDataと同じ作り
void oledBitMap(int offsetX , int offsetY , int width,int hight, const uint8_t bitMap[]) {
  int pos = 0 ;
  for (int m = 0; m < hight; m++) {
    oledSendCommand3(0xB0 | ((m + offsetY) & 0x07) , 0x10 | (offsetX & 0xf0) >> 4 , offsetX & 0x0F) ;

    Wire.beginTransmission(oledAddr);
    Wire.write(oldData);
    for (int x = 0; x < width; x++) {
      int8_t byteData = pgm_read_word(bitMap + pos) ;
      if (Wire.write(byteData) == 0) {
        Wire.endTransmission();
        Wire.beginTransmission(oledAddr);
        Wire.write(oldData);
        Wire.write(byteData);
      }
      pos++ ;
    }
    Wire.endTransmission();
  }
  oledSendCommand3(0xB0 , 0x10 , 0x00) ;
}

// ----------------------
// ----- セットアップ -----
// ----------------------
void setup() {
  Wire.begin();

  // OLED設定コマンド送信
  oledSendCommands(sizeof(oledInitCommands), oledInitCommands) ;  // 初期化
  oledFillData(0xAA) ;    // 画面塗りつぶし
  oledBitMap(20 , 0 , 88 , 8, bitmapData); // Bitmap表示
  oledSendCommand(0xAF) ; // 画面表示
}

void loop() {
  // put your main code here, to run repeatedly:
  oledSendCommand(0xA5) ;
  oledFillData(0xAA) ;    // 画面塗りつぶし
  oledBitMap(20 , 0 , 88 , 8, bitmapData); // Bitmap表示
  oledSendCommand(0xA4) ;
  delay(2000) ;

  oledSendCommand(0xA5) ;
  oledFillData(0xAA) ;    // 画面塗りつぶし
  oledBitMap(20 , 0 , 96 , 8, sasara); // Bitmap表示
  oledSendCommand(0xA4) ;
  delay(2000) ;

  oledSendCommand(0xA5) ;
  oledFillData(0xAA) ;    // 画面塗りつぶし
  oledBitMap(10 , 0 , 112 , 8, tudumi); // Bitmap表示
  oledSendCommand(0xA4) ;
  delay(2000) ;
}
